from I3Tray import *
from icecube import icetray, phys_services, dataio, dataclasses,MuonGun
from icecube.simprod import segments
import sys, math, numpy
import copy
import glob

import os

import time
from icecube.hdfwriter import I3HDFTableService, I3HDFWriter

import argparse


start_time = time.asctime()
print ('Started:', start_time)

# handling of command line arguments  
from optparse import OptionParser
parser = OptionParser()
usage = """%prog [options]"""
parser.set_usage(usage)
parser.add_option("-i", "--input", action="store", type="string", default="/data/ana/MESE/NuGen/SnowStorm/L3/L3_NewIce_022018_016000.i3.zst", dest="INPUT", help="Input i3 file")                 
parser.add_option("-o", "--output", action="store", type="string", default="/home/vbasu/testoutput/DNN", dest="OUTPUT", help="Output i3 file")

parser.add_option("-g", "--gcd", action="store", type="string", default="/cvmfs/icecube.opensciencegrid.org/data/GCD/GeoCalibDetectorStatus_AVG_55697-57531_PASS2_SPE_withScaledNoise.i3.gz", dest="GCD", help="GCD file for input i3 file")

#parser.add_option("-f","--grid", action="store", type="int", default=0, dest="GridFTP", help="to use gridftp or not. 0 is false and 1 is true")

# parse cmd line args, bail out if anything is not understood
(options,args) = parser.parse_args()
infile = options.INPUT
outfile = options.OUTPUT
gcdFile=options.GCD
tray = I3Tray()
##########################################################################################################
##functions
##########################################################################################################

##functions required to see if a particle is within the detector volume + 300m
import matplotlib.path as mpltPath
def get_surface_det():
    
    from icecube import MuonGun
    
    bound_2D=[]
    MuonGunGCD='/cvmfs/icecube.opensciencegrid.org/data/GCD/GeoCalibDetectorStatus_AVG_55697-57531_PASS2_SPE_withScaledNoise.i3.gz'
    surface_det = MuonGun.ExtrudedPolygon.from_file(MuonGunGCD, padding=300)##Build Polygon from I3Geometry
    
    x=[(surface_det.x[i],surface_det.y[i])for i in range(len(surface_det.x))]###getting only x and y
    bound_2D=mpltPath.Path(x)#Projection of detector on x,y plane
    return bound_2D, surface_det

def boundary_check(particle):
    ####checks if particle is inside the detector###
    bound_2D,surface_det = get_surface_det()
    inlimit = False  
    if ((particle.pos.z <=max(surface_det.z)) and (particle.pos.z>=min(surface_det.z))):
        if bound_2D.contains_points([(particle.pos.x, particle.pos.y)]):
            inlimit=True

    return inlimit

##gets the deposited energy of Truth from the mctree
def get_deposit_energy(mctree):
    losses = 0
    for p in mctree:
        if not p.is_cascade: continue
        if not p.location_type == dataclasses.I3Particle.InIce: continue
        if p.shape == p.Dark: continue
        if p.type in [p.Hadrons, p.PiPlus, p.PiMinus, p.NuclInt]:
            #hadlosses += p.energy
            if p.energy < 1*I3Units.GeV:
                losses += 0.8*p.energy
            else:
                energyScalingFactor = 1.0 + ((p.energy/I3Units.GeV/0.399)**-0.130)*(0.467 - 1)
                losses += energyScalingFactor*p.energy
        else:
            #emlosses += p.energy
            losses += p.energy

    return losses



##grabs highest energy hadron in boundary, and deposited energy
def get_highest_inice_params(frame):
    mctree = frame['I3MCTree']
    max_e = 0
    max_p = None
    depo_start = get_deposit_energy(mctree)
    depo_energy = 0
    for p in mctree:
        if boundary_check(p) == True and p.type in [dataclasses.I3Particle.Hadrons]:
            if p.energy > max_e:
                max_e = p.energy
                max_p = p
            depo_energy = depo_start
    if max_p is None:
        print(mctree)
        print(frame['I3EventHeader'])
        return False
    frame["highest_inice_depo_energy"]=dataclasses.I3Double(depo_energy)
    frame["highest_inice_hadron"]=dataclasses.I3Particle(max_p)



tray.AddSegment(dataio.I3Reader, 'reader', FilenameList=[options.GCD, infile])
tray.Add(get_highest_inice_params)

##new keys to add to i3/hdf5 file
table_keys = ["highest_inice_depo_energy","highest_inice_hadron"]



##writes hdf5 file
tray.Add(I3HDFWriter,'HDFwriter',
               Output=outfile+".hdf5",
               keys         = table_keys,
               SubEventStreams = ['topological_split','InIceSplit']
)

print(outfile)
# writes the i3 file
tray.AddModule('I3Writer', 'writer',
    DropOrphanStreams=[icetray.I3Frame.DAQ],
    Streams=[  icetray.I3Frame.DAQ, icetray.I3Frame.Physics],
    filename=outfile+".i3.zst")

tray.AddModule('TrashCan', 'thecan')

tray.Execute()
tray.Finish()


del tray

stop_time = time.asctime()

print ('Started:', start_time)
print ('Ended:', stop_time)